# encoding: utf-8
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.

module Azure::Web::Mgmt::V2016_06_01
  #
  # ConnectionGatewayInstallations
  #
  class ConnectionGatewayInstallations
    include MsRestAzure

    #
    # Creates and initializes a new instance of the ConnectionGatewayInstallations class.
    # @param client service class for accessing basic functionality.
    #
    def initialize(client)
      @client = client
    end

    # @return [LogicAppsManagementClient] reference to the LogicAppsManagementClient
    attr_reader :client

    #
    # Gets a list of installed gateways that the user is an admin of
    #
    # Gets a list of installed gateways that the user is an admin of, in a specific
    # subscription and at a certain location
    #
    # @param subscription_id [String] Subscription Id
    # @param location [String] The location
    # @param api_version [String] API Version
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ConnectionGatewayInstallationDefinitionCollection] operation
    # results.
    #
    def list(subscription_id, location, api_version, custom_headers:nil)
      response = list_async(subscription_id, location, api_version, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets a list of installed gateways that the user is an admin of
    #
    # Gets a list of installed gateways that the user is an admin of, in a specific
    # subscription and at a certain location
    #
    # @param subscription_id [String] Subscription Id
    # @param location [String] The location
    # @param api_version [String] API Version
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_with_http_info(subscription_id, location, api_version, custom_headers:nil)
      list_async(subscription_id, location, api_version, custom_headers:custom_headers).value!
    end

    #
    # Gets a list of installed gateways that the user is an admin of
    #
    # Gets a list of installed gateways that the user is an admin of, in a specific
    # subscription and at a certain location
    #
    # @param subscription_id [String] Subscription Id
    # @param location [String] The location
    # @param api_version [String] API Version
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_async(subscription_id, location, api_version, custom_headers:nil)
      fail ArgumentError, 'subscription_id is nil' if subscription_id.nil?
      fail ArgumentError, 'location is nil' if location.nil?
      fail ArgumentError, 'api_version is nil' if api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.Web/locations/{location}/connectionGatewayInstallations'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => subscription_id,'location' => location},
          query_params: {'api-version' => api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::Web::Mgmt::V2016_06_01::Models::ConnectionGatewayInstallationDefinitionCollection.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Gets an installed gateway that the user is an admin of
    #
    # Get a specific installed gateway that the user is an admin of, in a specific
    # subscription and at a certain location
    #
    # @param subscription_id [String] Subscription Id
    # @param location [String] The location
    # @param gateway_id [String] Gateway ID
    # @param api_version [String] API Version
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ConnectionGatewayInstallationDefinition] operation results.
    #
    def get(subscription_id, location, gateway_id, api_version, custom_headers:nil)
      response = get_async(subscription_id, location, gateway_id, api_version, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets an installed gateway that the user is an admin of
    #
    # Get a specific installed gateway that the user is an admin of, in a specific
    # subscription and at a certain location
    #
    # @param subscription_id [String] Subscription Id
    # @param location [String] The location
    # @param gateway_id [String] Gateway ID
    # @param api_version [String] API Version
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def get_with_http_info(subscription_id, location, gateway_id, api_version, custom_headers:nil)
      get_async(subscription_id, location, gateway_id, api_version, custom_headers:custom_headers).value!
    end

    #
    # Gets an installed gateway that the user is an admin of
    #
    # Get a specific installed gateway that the user is an admin of, in a specific
    # subscription and at a certain location
    #
    # @param subscription_id [String] Subscription Id
    # @param location [String] The location
    # @param gateway_id [String] Gateway ID
    # @param api_version [String] API Version
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def get_async(subscription_id, location, gateway_id, api_version, custom_headers:nil)
      fail ArgumentError, 'subscription_id is nil' if subscription_id.nil?
      fail ArgumentError, 'location is nil' if location.nil?
      fail ArgumentError, 'gateway_id is nil' if gateway_id.nil?
      fail ArgumentError, 'api_version is nil' if api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.Web/locations/{location}/connectionGatewayInstallations/{gatewayId}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => subscription_id,'location' => location,'gatewayId' => gateway_id},
          query_params: {'api-version' => api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::Web::Mgmt::V2016_06_01::Models::ConnectionGatewayInstallationDefinition.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

  end
end
